/* IRCEvents.h - NeonServ v5.5
 * Copyright (C) 2011-2012  Philipp Kreil (pk910)
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program. If not, see <http://www.gnu.org/licenses/>. 
 */
#ifndef _IRCEvents_h
#define _IRCEvents_h

#include "main.h"

struct UserNode;
struct ChanNode;
struct ChanUser;
struct ClientSocket;

#ifndef DND_FUNCTIONS
void init_bind();
void free_bind();
void unregister_module_events(int module_id);
#endif

typedef void join_func_t(struct ChanUser *chanuser);
#ifndef DND_FUNCTIONS
/* MODULAR ACCESSIBLE */ int bind_join(join_func_t *func, int module_id);
/* MODULAR ACCESSIBLE */ void unbind_join(join_func_t *func);
int event_join(struct ChanUser *chanuser);
#endif

typedef void nick_func_t(struct UserNode *user, char *new_nick);
#ifndef DND_FUNCTIONS
/* MODULAR ACCESSIBLE */ int bind_nick(nick_func_t *func, int module_id);
/* MODULAR ACCESSIBLE */ void unbind_nick(nick_func_t *func);
int event_nick(struct UserNode *user, char *new_nick);
#endif

typedef void part_func_t(struct ChanUser *chanuser, int quit, char *reason);
#ifndef DND_FUNCTIONS
/* MODULAR ACCESSIBLE */ int bind_part(part_func_t *func, int module_id);
/* MODULAR ACCESSIBLE */ void unbind_part(part_func_t *func);
int event_part(struct ChanUser *chanuser, int quit, char *reason);
#endif

typedef void kick_func_t(struct UserNode *user, struct ChanUser *target, char *reason);
#ifndef DND_FUNCTIONS
/* MODULAR ACCESSIBLE */ int bind_kick(kick_func_t *func, int module_id);
/* MODULAR ACCESSIBLE */ void unbind_kick(kick_func_t *func);
int event_kick(struct UserNode *user, struct ChanUser *target, char *reason);
#endif

typedef void topic_func_t(struct UserNode *user, struct ChanNode *chan, const char *new_topic);
#ifndef DND_FUNCTIONS
/* MODULAR ACCESSIBLE */ int bind_topic(topic_func_t *func, int module_id);
/* MODULAR ACCESSIBLE */ void unbind_topic(topic_func_t *func);
int event_topic(struct UserNode *user, struct ChanNode *chan, const char *new_topic);
#endif

typedef void mode_func_t(struct UserNode *user, struct ChanNode *chan, char *modes, char **argv, int argc);
#ifndef DND_FUNCTIONS
/* MODULAR ACCESSIBLE */ int bind_mode(mode_func_t *func, int module_id);
/* MODULAR ACCESSIBLE */ void unbind_mode(mode_func_t *func);
int event_mode(struct UserNode *user, struct ChanNode *chan, char *modes, char **argv, int argc);
#endif

typedef void chanmsg_func_t(struct UserNode *user, struct ChanNode *chan, char *message);
#ifndef DND_FUNCTIONS
/* MODULAR ACCESSIBLE */ int bind_chanmsg(chanmsg_func_t *func, int module_id);
/* MODULAR ACCESSIBLE */ void unbind_chanmsg(chanmsg_func_t *func);
int event_chanmsg(struct UserNode *user, struct ChanNode *chan, char *message);
#endif

typedef void privmsg_func_t(struct UserNode *user, struct UserNode *target, char *message);
#ifndef DND_FUNCTIONS
/* MODULAR ACCESSIBLE */ int bind_privmsg(privmsg_func_t *func, int module_id);
/* MODULAR ACCESSIBLE */ void unbind_privmsg(privmsg_func_t *func);
int event_privmsg(struct UserNode *user, struct UserNode *target, char *message);
#endif

typedef void channotice_func_t(struct UserNode *user, struct ChanNode *chan, char *message);
#ifndef DND_FUNCTIONS
/* MODULAR ACCESSIBLE */ int bind_channotice(channotice_func_t *func, int module_id);
/* MODULAR ACCESSIBLE */ void unbind_channotice(channotice_func_t *func);
int event_channotice(struct UserNode *user, struct ChanNode *chan, char *message);
#endif

typedef void privnotice_func_t(struct UserNode *user, struct UserNode *target, char *message);
#ifndef DND_FUNCTIONS
/* MODULAR ACCESSIBLE */ int bind_privnotice(privnotice_func_t *func, int module_id);
/* MODULAR ACCESSIBLE */ void unbind_privnotice(privnotice_func_t *func);
int event_privnotice(struct UserNode *user, struct UserNode *target, char *message);
#endif

typedef void chanctcp_func_t(struct UserNode *user, struct ChanNode *chan, char *command, char *text);
#ifndef DND_FUNCTIONS
/* MODULAR ACCESSIBLE */ int bind_chanctcp(chanctcp_func_t *func, int module_id);
/* MODULAR ACCESSIBLE */ void unbind_chanctcp(chanctcp_func_t *func);
int event_chanctcp(struct UserNode *user, struct ChanNode *chan, char *command, char *text);
#endif

typedef void privctcp_func_t(struct UserNode *user, struct UserNode *target, char *command, char *text);
#ifndef DND_FUNCTIONS
/* MODULAR ACCESSIBLE */ int bind_privctcp(privctcp_func_t *func, int module_id);
/* MODULAR ACCESSIBLE */ void unbind_privctcp(privctcp_func_t *func);
int event_privctcp(struct UserNode *user, struct UserNode *target, char *command, char *text);
#endif

typedef void invite_func_t(struct ClientSocket *client, struct UserNode *user, char *channel);
#ifndef DND_FUNCTIONS
/* MODULAR ACCESSIBLE */ int bind_invite(invite_func_t *func, int module_id);
/* MODULAR ACCESSIBLE */ void unbind_invite(invite_func_t *func);
int event_invite(struct ClientSocket *client, struct UserNode *user, char *channel);
#endif

typedef void raw_func_t(struct ClientSocket *client, char *from, char *cmd, char **argv, int argc);
#ifndef DND_FUNCTIONS
/* MODULAR ACCESSIBLE */ int bind_raw(raw_func_t *func, int module_id);
/* MODULAR ACCESSIBLE */ void unbind_raw(raw_func_t *func);
int event_raw(struct ClientSocket *client, char *from, char *cmd, char **argv, int argc);
#endif

typedef void bot_ready_func_t(struct ClientSocket *client);
#ifndef DND_FUNCTIONS
/* MODULAR ACCESSIBLE */ int bind_bot_ready(bot_ready_func_t *func, int module_id);
/* MODULAR ACCESSIBLE */ void unbind_bot_ready(bot_ready_func_t *func);
int event_bot_ready(struct ClientSocket *client);
#endif

typedef void registered_func_t(struct UserNode *user, char *new_mask);
#ifndef DND_FUNCTIONS
/* MODULAR ACCESSIBLE */ int bind_registered(registered_func_t *func, int module_id);
/* MODULAR ACCESSIBLE */ void unbind_registered(registered_func_t *func);
int event_registered(struct UserNode *user, char *new_mask);
#endif

typedef int freeuser_func_t(struct UserNode *user);
#ifndef DND_FUNCTIONS
/* MODULAR ACCESSIBLE */ int bind_freeuser(freeuser_func_t *func, int module_id);
/* MODULAR ACCESSIBLE */ void unbind_freeuser(freeuser_func_t *func);
int event_freeuser(struct UserNode *user);
#endif

typedef int freechan_func_t(struct ChanNode *chan);
#ifndef DND_FUNCTIONS
/* MODULAR ACCESSIBLE */ int bind_freechan(freechan_func_t *func, int module_id);
/* MODULAR ACCESSIBLE */ void unbind_freechan(freechan_func_t *func);
int event_freechan(struct ChanNode *chan);
#endif

typedef int reload_func_t(int initialization);
#ifndef DND_FUNCTIONS
/* MODULAR ACCESSIBLE */ int bind_reload(reload_func_t *func, int module_id);
/* MODULAR ACCESSIBLE */ void unbind_reload(reload_func_t *func);
int event_reload(int initialization);
#endif

#endif
