/* ModeNode.h - NeonServ v5.5
 * Copyright (C) 2011-2012  Philipp Kreil (pk910)
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program. If not, see <http://www.gnu.org/licenses/>. 
 */
#ifndef _ModeNode_h
#define _ModeNode_h
#include "main.h"

struct ChanNode;

//Types: http://www.irc.org/tech_docs/draft-brocklesby-irc-isupport-03.txt
#define CHANNEL_MODE_TYPE_A        0x01 /* ... special (addresses or users) ... */
#define CHANNEL_MODE_TYPE_B        0x02 /* These modes always take a parameter. */
#define CHANNEL_MODE_TYPE_C        0x03 /* These modes take a parameter only when set. */
#define CHANNEL_MODE_TYPE_D        0x04 /* These modes never take a parameter. */
#define CHANNEL_MODE_TYPE          0x07 /* bit mask to get the type */

#define CHANNEL_MODE_VALUE_STRING  0x10
#define CHANNEL_MODE_VALUE_INTEGER 0x20
#define CHANNEL_MODE_VALUE         0x30 /* bit mask to get the value */

#define CHANNEL_MODE_KEY           0x40 /* mode is a key - automatically add the current key as parameter on unset */

#define CHANNEL_MODE_VALUE_INDEX_SHIFT 8
#define CHANNEL_MODE_VALUE_INDEX_MASK  (0xff << CHANNEL_MODE_VALUE_INDEX_SHIFT) /* this "bitrange" is reserved for storing the array indexes of the mode values */

struct ModeNode {
    struct ChanNode *chan;
    unsigned int modes;
    unsigned int allmodes;
    char **mode_str_args;
    int *mode_int_args;
};

#ifndef DND_FUNCTIONS
void init_ModeNode();
/* MODULAR ACCESSIBLE */ struct ModeNode *createModeNode(struct ChanNode *chan);
/* MODULAR ACCESSIBLE */ void freeModeNode(struct ModeNode *modes);
/* MODULAR ACCESSIBLE */ int isModeSet(struct ModeNode* modes, char modeChar);
/* MODULAR ACCESSIBLE */ int isModeAffected(struct ModeNode* modes, char modeChar);
/* MODULAR ACCESSIBLE */ void* getModeValue(struct ModeNode* modes, char modeChar);
/* MODULAR ACCESSIBLE */ unsigned int getModeType(struct ModeNode* modes, char modeChar);
/* MODULAR ACCESSIBLE */ void parseModes(struct ModeNode* modes, char *modeStr, char **argv, int argc);
/* MODULAR ACCESSIBLE */ void parseModeString(struct ModeNode* modes, char *modeStr);
/* MODULAR ACCESSIBLE */ int parseMode(struct ModeNode* modes, int add, char mode, char *param);
/* MODULAR ACCESSIBLE */ void getModeString(struct ModeNode* modes, char *modesStr);
/* MODULAR ACCESSIBLE */ void getFullModeString(struct ModeNode* modes, char *modesStr);
#endif
#endif
