/* modcmd.h - NeonServ v5.5
 * Copyright (C) 2011-2012  Philipp Kreil (pk910)
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program. If not, see <http://www.gnu.org/licenses/>. 
 */
#ifndef _modcmd_h
#define _modcmd_h
#include "main.h"

#define MAXPARAMETERS 50

#define CMDFLAG_REQUIRE_CHAN            0x0001
#define CMDFLAG_REQUIRE_AUTH            0x0002
#define CMDFLAG_REQUIRE_GOD             0x0004
#define CMDFLAG_CHECK_AUTH              0x0008
#define CMDFLAG_REGISTERED_CHAN         0x0010
#define CMDFLAG_OVERRIDE_GLOBAL_ACCESS  0x0020
#define CMDFLAG_OVERRIDE_CHANNEL_ACCESS 0x0040
#define CMDFLAG_CHAN_PARAM              0x0080
#define CMDFLAG_LOG                     0x0100
#define CMDFLAG_OPLOG                   0x0200
#define CMDFLAG_EMPTY_ARGS              0x0400
#define CMDFLAG_REQUIRED                0x0800
#define CMDFLAG_TEMPONARY_BIND          0x1000
#define CMDFLAG_FUNCMD                  0x2000
#define CMDFLAG_ESCAPE_ARGS             0x4000
#define CMDFLAG_NO_CROSSCHAN            0x8000

struct ClientSocket;
struct UserNode;
struct ChanNode;
struct Event;

#define CMD_BIND(NAME) void NAME(UNUSED_ARG(struct ClientSocket *client), UNUSED_ARG(struct ClientSocket *textclient), UNUSED_ARG(struct UserNode *user), UNUSED_ARG(struct ChanNode *chan), UNUSED_ARG(char **argv), UNUSED_ARG(char argc), UNUSED_ARG(struct Event *event))
typedef void cmd_bind_t(struct ClientSocket *client, struct ClientSocket *textclient, struct UserNode *user, struct ChanNode *chan, char **argv, char argc, struct Event *event);
typedef void trigger_callback_t(int clientid, struct ChanNode *chan, char *trigger);

struct cmd_function {
    char *name;
    int botid;
    int module_id;
    cmd_bind_t *func;
    unsigned int flags;
    int paramcount;
    int global_access;
    char *channel_access;
    
    struct cmd_function *next;
};

struct cmd_binding {
    char *cmd;
    int botid;
    int clientid;
    struct cmd_function *func;
    unsigned int flags;
    char *parameters[MAXPARAMETERS];
    int paramcount;
    int global_access;
    char *channel_access;
    int triggered;
    
    struct cmd_binding *next;
};

#define BIND_FLAGS(BIND) (BIND->flags | BIND->func->flags)

struct trigger_cache {
    int botid;
    int clientid;
    char *trigger;
    
    struct trigger_cache *next;
};

#define get_prefered_bot(BOTID) get_botwise_prefered_bot(BOTID, 0)
#define changeChannelTrigger(BOTID,CHAN,NEW) changeBotwiseChannelTrigger(BOTID, 0, CHAN, NEW)
#define bind_cmd_to_function(BOTID,CMD,FUNC) bind_botwise_cmd_to_function(BOTID, 0, CMD, FUNC)
#define bind_cmd_to_command(BOTID,CMD,FUNC) bind_botwise_cmd_to_command(BOTID, 0, CMD, FUNC)
#define unbind_cmd(BOTID,CMD) unbind_botwise_cmd(BOTID, 0, CMD)
#define unbind_allcmd(BOTID) unbind_botwise_allcmd(BOTID, 0)
#define bind_set_parameters(BOTID,CMD,PARAMETERS) bind_botwise_set_parameters(BOTID, 0, CMD, PARAMETERS)
#define bind_set_global_access(BOTID,CMD,GACCESS) bind_botwise_set_global_access(BOTID, 0, CMD, GACCESS)
#define bind_set_channel_access(BOTID,CMD,CACCESS) bind_botwise_set_channel_access(BOTID, 0, CMD, CACCESS)
#define bind_set_bind_flags(BOTID,CMD,FLAGS) bind_botwise_set_bind_flags(BOTID, 0, CMD, FLAGS)
#define find_cmd_binding(BOTID,CMD) find_botwise_cmd_binding(BOTID, 0, CMD)
#define bind_unbound_required_functions(BOTID) bind_botwise_unbound_required_functions(BOTID, 0)


#ifndef DND_FUNCTIONS
extern int statistics_commands;

void init_modcmd();
void free_modcmd();

/* MODULAR ACCESSIBLE */ struct ClientSocket* get_botwise_prefered_bot(int botid, int clientid);

/* MODULAR ACCESSIBLE */ int register_command(int botid, char *name, int module_id, cmd_bind_t *func, int paramcount, char *channel_access, int global_access, unsigned int flags);
/* MODULAR ACCESSIBLE */ int set_trigger_callback(int botid, int module_id, trigger_callback_t *func);

/* MODULAR ACCESSIBLE */ int flush_trigger_cache(int botid, int clientid);
/* MODULAR ACCESSIBLE */ int changeBotwiseChannelTrigger(int botid, int clientid, struct ChanNode *chan, char *new_trigger);
/* MODULAR ACCESSIBLE */ int bind_botwise_cmd_to_function(int botid, int clientid, char *cmd, struct cmd_function *func);
/* MODULAR ACCESSIBLE */ int bind_botwise_cmd_to_command(int botid, int clientid, char *cmd, char *func);
/* MODULAR ACCESSIBLE */ int unbind_botwise_cmd(int botid, int clientid, char *cmd);
/* MODULAR ACCESSIBLE */ int unbind_botwise_allcmd(int botid, int clientid);
/* MODULAR ACCESSIBLE */ void bind_botwise_set_parameters(int botid, int clientid, char *cmd, char *parameters);
/* MODULAR ACCESSIBLE */ void bind_botwise_set_global_access(int botid, int clientid, char *cmd, int gaccess);
/* MODULAR ACCESSIBLE */ void bind_botwise_set_channel_access(int botid, int clientid, char *cmd, char *chanaccess);
/* MODULAR ACCESSIBLE */ void bind_botwise_set_bind_flags(int botid, int clientid, char *cmd, unsigned int flags);
/* MODULAR ACCESSIBLE */ struct cmd_binding *find_botwise_cmd_binding(int botid, int clientid, char *cmd);
/* MODULAR ACCESSIBLE */ void bind_botwise_unbound_required_functions(int botid, int clientid);
/* MODULAR ACCESSIBLE */ struct cmd_function *find_cmd_function(int botid, char *name);
/* MODULAR ACCESSIBLE */ void register_command_alias(int botid, char *alias);
/* MODULAR ACCESSIBLE */ struct cmd_binding *getAllBinds(struct cmd_binding *last);

void unregister_module_commands(int module_id);

#endif
#endif
