/* modules.c - NeonServ v5.5
 * Copyright (C) 2011-2012  Philipp Kreil (pk910)
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program. If not, see <http://www.gnu.org/licenses/>. 
 */
#include "modules.h"
#ifndef WIN32
#include <dlfcn.h>
#endif

/* 000-011 */ #include "main.h"
/* 012     */ #include "BanNode.h"
/* 013-019 */ #include "bots.h"
/* 020-025 */ #include "ChanNode.h"
/* 026-029 */ #include "ChanUser.h"
/* 030-036 */ #include "ClientSocket.h"
/* 037-038 */ #include "ConfigParser.h"
/* 039-048 */ #include "DBHelper.h"
/* 049     */ #include "EventLogger.h"
/* 050     */ #include "HandleInfoHandler.h"
/* 051-088 */ #include "IRCEvents.h"
/* 089-091 */ #include "IRCParser.h"
/* 092-098 */ #include "lang.h"
#ifdef ENABLE_MEMORY_DEBUG
/* 099-102 */ #include "memoryDebug.h"
#endif
/* 103-106 */ #include "memoryInfo.h"
/* 107-125 */ #include "modcmd.h"
/* 126-136 */ #include "ModeNode.h"
/* 137-142 */ #include "mysqlConn.h"
/* 143-149 */ #include "timeq.h"
/* 150-169 */ #include "tools.h"
/* 170-180 */ #include "UserNode.h"
/* 181-183 */ #include "WHOHandler.h"
/* 184-188 */ #include "version.h"
/* 189     */ /* modules.h */
/* 190     */ /* UserNode.h */
/* 191-193 */ #include "ModuleFunctions.h"
/* 194     */ /* bots.h */

#define Function void *

void *global_functions[] = {
/* 000 */ (Function) getStartTime,
/* 001 */ (Function) getRunningThreads,
/* 002 */ (Function) exit_daemon,
/* 003 */ (Function) stricmp,
/* 004 */ (Function) stricmplen,
/* 005 */ (Function) restart_process,
/* 006 */ (Function) cleanup,
/* 007 */ (Function) restart_bot,
/* 008 */ (Function) stop_bot,
/* 009 */ (Function) reload_config,
/* 010 */ (Function) putlog,
#ifdef HAVE_THREADS
/* 011 */ (Function) getCurrentThreadID,
#else
/* 011 */ (Function) NULL,
#endif
/* 012 */ (Function) getMatchingChannelBan,
/* 013 */ (Function) getChannelBot,
/* 014 */ (Function) requestOp,
/* 015 */ (Function) channel_ban_timeout,
/* 016 */ (Function) general_event_privctcp,
/* 017 */ (Function) set_bot_alias,
/* 018 */ (Function) resolve_botid,
/* 019 */ (Function) resolve_botalias,
/* 020 */ (Function) is_valid_chan,
/* 021 */ (Function) getAllChans,
/* 022 */ (Function) getChanByName,
/* 023 */ (Function) getChannelCount,
/* 024 */ (Function) getChanUserCount,
/* 025 */ (Function) getChanBanCount,
/* 026 */ (Function) isUserOnChan,
/* 027 */ (Function) getChanUser,
/* 028 */ (Function) getChannelUsers,
/* 029 */ (Function) getUserChannels,
/* 030 */ (Function) create_socket,
/* 031 */ (Function) connect_socket,
/* 032 */ (Function) close_socket,
/* 033 */ (Function) destroy_socket,
/* 034 */ (Function) write_socket,
/* 035 */ (Function) putsock,
/* 036 */ (Function) getBots,
/* 037 */ (Function) get_int_field,
/* 038 */ (Function) get_string_field,
/* 039 */ (Function) _loadUserSettings,
/* 040 */ (Function) isGodMode,
/* 041 */ (Function) getChanDefault,
/* 042 */ (Function) getChannelAccess,
/* 043 */ (Function) checkChannelAccess,
/* 044 */ (Function) _loadChannelSettings,
/* 045 */ (Function) isUserProtected,
/* 046 */ (Function) getBanAffectingMask,
/* 047 */ (Function) renameAccount,
/* 048 */ (Function) deleteUser,
/* 049 */ (Function) logEvent,
/* 050 */ (Function) lookup_authname,
/* 051 */ (Function) bind_join,
/* 052 */ (Function) unbind_join,
/* 053 */ (Function) bind_nick,
/* 054 */ (Function) unbind_nick,
/* 055 */ (Function) bind_part,
/* 056 */ (Function) unbind_part,
/* 057 */ (Function) bind_reload,
/* 058 */ (Function) unbind_reload,
/* 059 */ (Function) bind_kick,
/* 060 */ (Function) unbind_kick,
/* 061 */ (Function) bind_topic,
/* 062 */ (Function) unbind_topic,
/* 063 */ (Function) bind_mode,
/* 064 */ (Function) unbind_mode,
/* 065 */ (Function) bind_chanmsg,
/* 066 */ (Function) unbind_chanmsg,
/* 067 */ (Function) bind_privmsg,
/* 068 */ (Function) unbind_privmsg,
/* 069 */ (Function) bind_channotice,
/* 070 */ (Function) unbind_channotice,
/* 071 */ (Function) bind_privnotice,
/* 072 */ (Function) unbind_privnotice,
/* 073 */ (Function) bind_chanctcp,
/* 074 */ (Function) unbind_chanctcp,
/* 075 */ (Function) bind_privctcp,
/* 076 */ (Function) unbind_privctcp,
/* 077 */ (Function) bind_invite,
/* 078 */ (Function) unbind_invite,
/* 079 */ (Function) bind_raw,
/* 080 */ (Function) unbind_raw,
/* 081 */ (Function) bind_bot_ready,
/* 082 */ (Function) unbind_bot_ready,
/* 083 */ (Function) bind_registered,
/* 084 */ (Function) unbind_registered,
/* 085 */ (Function) bind_freeuser,
/* 086 */ (Function) unbind_freeuser,
/* 087 */ (Function) bind_freechan,
/* 088 */ (Function) unbind_freechan,
/* 089 */ (Function) reply,
/* 090 */ (Function) merge_argv,
/* 091 */ (Function) merge_argv_char,
/* 092 */ (Function) get_language_by_tag,
/* 093 */ (Function) get_language_by_name,
/* 094 */ (Function) get_default_language,
/* 095 */ (Function) load_language,
/* 096 */ (Function) register_default_language_table,
/* 097 */ (Function) get_language_string,
/* 098 */ (Function) build_language_string,
#ifdef ENABLE_MEMORY_DEBUG
/* 099 */ (Function) xmalloc,
/* 100 */ (Function) xcalloc,
/* 101 */ (Function) xstrdup,
/* 102 */ (Function) xfree,
#else
/* 099 */ (Function) NULL,
/* 100 */ (Function) NULL,
/* 101 */ (Function) NULL,
/* 102 */ (Function) NULL,
#endif
/* 103 */ (Function) getMemoryInfoFiles,
/* 104 */ (Function) freeMemoryInfoFiles,
/* 105 */ (Function) getMemoryInfoLines,
/* 106 */ (Function) freeMemoryInfoLines,
/* 107 */ (Function) get_botwise_prefered_bot,
/* 108 */ (Function) register_command,
/* 109 */ (Function) set_trigger_callback,
/* 110 */ (Function) flush_trigger_cache,
/* 111 */ (Function) changeBotwiseChannelTrigger,
/* 112 */ (Function) bind_botwise_cmd_to_function,
/* 113 */ (Function) bind_botwise_cmd_to_command,
/* 114 */ (Function) unbind_botwise_cmd,
/* 115 */ (Function) unbind_botwise_allcmd,
/* 116 */ (Function) bind_botwise_set_parameters,
/* 117 */ (Function) bind_botwise_set_global_access,
/* 118 */ (Function) bind_botwise_set_channel_access,
/* 119 */ (Function) bind_botwise_set_bind_flags,
/* 120 */ (Function) find_botwise_cmd_binding,
/* 121 */ (Function) bind_botwise_unbound_required_functions,
/* 122 */ (Function) find_cmd_function,
/* 123 */ (Function) NULL, /* deprecated */
/* 124 */ (Function) register_command_alias,
/* 125 */ (Function) getAllBinds,
/* 126 */ (Function) createModeNode,
/* 127 */ (Function) freeModeNode,
/* 128 */ (Function) isModeSet,
/* 129 */ (Function) isModeAffected,
/* 130 */ (Function) getModeValue,
/* 131 */ (Function) getModeType,
/* 132 */ (Function) parseModes,
/* 133 */ (Function) parseModeString,
/* 134 */ (Function) parseMode,
/* 135 */ (Function) getModeString,
/* 136 */ (Function) getFullModeString,
/* 137 */ (Function) mysql_use,
/* 138 */ (Function) mysql_free,
/* 139 */ (Function) printf_mysql_query,
/* 140 */ (Function) printf_long_mysql_query,
/* 141 */ (Function) escape_string,
/* 142 */ (Function) get_mysql_conn,
/* 143 */ (Function) timeq_add,
/* 144 */ (Function) timeq_uadd,
/* 145 */ (Function) timeq_add_name,
/* 146 */ (Function) timeq_uadd_name,
/* 147 */ (Function) timeq_del,
/* 148 */ (Function) timeq_del_name,
/* 149 */ (Function) timeq_name_exists,
/* 150 */ (Function) match,
/* 151 */ (Function) table_init,
/* 152 */ (Function) table_add,
/* 153 */ (Function) table_change,
/* 154 */ (Function) table_change_field,
/* 155 */ (Function) table_set_bold,
/* 156 */ (Function) table_end,
/* 157 */ (Function) table_free,
/* 158 */ (Function) timeToStr,
/* 159 */ (Function) strToTime,
/* 160 */ (Function) initModeBuffer,
/* 161 */ (Function) modeBufferSet,
/* 162 */ (Function) flushModeBuffer,
/* 163 */ (Function) freeModeBuffer,
/* 164 */ (Function) is_ircmask,
/* 165 */ (Function) generate_banmask,
/* 166 */ (Function) make_banmask,
/* 167 */ (Function) isFakeHost,
/* 168 */ (Function) mask_match,
/* 169 */ (Function) crc32,
/* 170 */ (Function) is_valid_nick,
/* 171 */ (Function) getUserByNick,
/* 172 */ (Function) getUserByMask,
/* 173 */ (Function) countUsersWithHost,
/* 174 */ (Function) getAuthFakehost,
/* 175 */ (Function) searchUserByNick,
/* 176 */ (Function) getAllUsers,
/* 177 */ (Function) getUsersWithAuth,
/* 178 */ (Function) getUserCount,
/* 179 */ (Function) createTempUser,
/* 180 */ (Function) createTempUserMask,
/* 181 */ (Function) get_userlist,
/* 182 */ (Function) _get_userlist_with_invisible,
/* 183 */ (Function) get_userauth,
/* 184 */ (Function) get_compilation,
/* 185 */ (Function) get_creation,
/* 186 */ (Function) get_revision,
/* 187 */ (Function) get_codelines,
/* 188 */ (Function) get_patchlevel,
/* 189 */ (Function) get_module_name,
/* 190 */ (Function) isUserModeSet,
/* 191 */ (Function) module_global_cmd_register_neonbackup,
/* 192 */ (Function) module_global_cmd_unregister_neonbackup,
/* 193 */ (Function) module_neonbackup_recover_chan,
/* 194 */ (Function) requestInvite
};

static int module_id_counter = 1;
static struct ModuleInfo *modules = NULL;

static void unregister_module_hooks(int module_id);

void loadModules() {
    char **modulelist = get_all_fieldnames("modules");
    if(!modulelist) return;
    int i = 0;
    char tmp[MAXLEN];
    struct ModuleInfo *modinfo;
    while(modulelist[i]) {
        sprintf(tmp, "modules.%s.enabled", modulelist[i]);
        if(get_int_field(tmp)) {
            modinfo = loadModule(modulelist[i]);
            sprintf(tmp, "modules.%s.protected", modulelist[i]);
            if(get_int_field(tmp))
                modinfo->state |= MODINFO_STATE_PROTECTED;
        }
        i++;
    }
    free(modulelist);
    start_modules();
}

struct ModuleInfo *loadModule(char *name) {
    struct ModuleInfo *modinfo;
    for(modinfo = modules; modinfo; modinfo = modinfo->next) {
        if(!stricmp(modinfo->name, name)) return NULL;
    }
    char fname[256];
    #ifndef WIN32
    sprintf(fname, "%s.so", name);
    void* module = dlopen(fname, RTLD_LAZY);
    if(!module) {
        sprintf(fname, "./%s.so", name);
        module = dlopen(fname, RTLD_LAZY);
    }
    if(!module) {
        sprintf(fname, ".libs/%s.so", name);
        module = dlopen(fname, RTLD_LAZY);
    }
    if(!module) {
        putlog(LOGLEVEL_ERROR, "Error loading module '%s': %s not found.\n", name, fname);
        return NULL;
    }
    void* initfunc = dlsym(module, "init_module");
    void* startfunc = dlsym(module, "start_module");
    void* stopfunc = dlsym(module, "stop_module");
    void* modversion = dlsym(module, "modversion");
    #else
    sprintf(fname, "%s.dll", name);
    HMODULE module = LoadLibrary(fname);
    if(!module) {
        putlog(LOGLEVEL_ERROR, "Error loading module '%s': %s not found.\n", name, fname);
        return NULL;
    }
    FARPROC initfunc = GetProcAddress(module, "init_module");
    FARPROC startfunc = GetProcAddress(module, "start_module");
    FARPROC stopfunc = GetProcAddress(module, "stop_module");
    FARPROC modversion = GetProcAddress(module, "modversion");
    #endif
    if(!startfunc || !stopfunc || !modversion) {
        putlog(LOGLEVEL_ERROR, "Error loading module '%s': required symbols not found.\n", name);
        return NULL;
    }
    int version = ((int (*)(void)) modversion)();
    if(version != MODULE_VERSION) {
        putlog(LOGLEVEL_ERROR, "Error loading module '%s': version mismatch ('%d' main code, '%d' module)\n", name, MODULE_VERSION, version);
        return NULL;
    }
    //start module
    int errid;
    int module_id = module_id_counter++;
    if((errid = ((int (*)(void **, int)) initfunc)(global_functions, module_id))) {
        putlog(LOGLEVEL_ERROR, "Error loading module '%s': module reported error (errid: %d)\n", name, errid);
        return NULL;
    }
    modinfo = malloc(sizeof(*modinfo));
    if(!modinfo) {
        unregister_module_hooks(module_id);
        return NULL;
    }
    modinfo->name = strdup(name);
    modinfo->module_id = module_id;
    modinfo->module = module;
    modinfo->startfunc = startfunc;
    modinfo->stopfunc = stopfunc;
    modinfo->state = 0;
    modinfo->next = modules;
    modules = modinfo;
    scan_module(modinfo);
    return modinfo;
}

#ifndef WIN32
static void closemodule(void *module) {
    dlclose(module);
}
#else
static void closemodule(HMODULE module) {
    FreeLibrary(module);
}
#endif

int ext_load_module(char *name) {
    if(!loadModule(name)) return 0;
    struct ModuleInfo *modinfo;
    for(modinfo = modules; modinfo; modinfo = modinfo->next) {
        if(!(modinfo->state & MODINFO_STATE_STARTED)) {
            modinfo->state |= MODINFO_STATE_STARTED;
            ((void (*)(int)) modinfo->startfunc)(MODSTATE_STARTSTOP);
        } else
            ((void (*)(int)) modinfo->startfunc)(MODSTATE_REBIND);
    }
    return 1;
}

int ext_unload_module(char *name) {
    struct ModuleInfo *old_modinfo, *old_prev = NULL;
    for(old_modinfo = modules; old_modinfo; old_modinfo = old_modinfo->next) {
        if(!stricmp(old_modinfo->name, name)) {
            if(old_modinfo->state & MODINFO_STATE_PROTECTED) {
                return 0;
            }
            if(old_prev)
                old_prev->next = old_modinfo->next;
            else
                modules = old_modinfo->next;
            unregister_module_hooks(old_modinfo->module_id);
            ((void (*)(int)) old_modinfo->stopfunc)(MODSTATE_STARTSTOP);
            closemodule(old_modinfo->module);
            free_module_functions(old_modinfo);
            free(old_modinfo->name);
            free(old_modinfo);
            return 1;
        } else
            old_prev = old_modinfo;
    }
    return 0;
}

int ext_reload_module(char *name) {
    char libname[256];
    struct ModuleInfo *old_modinfo, *old_prev = NULL;
    for(old_modinfo = modules; old_modinfo; old_modinfo = old_modinfo->next) {
        if(!stricmp(old_modinfo->name, name)) {
            strcpy(libname, old_modinfo->name);
            if(old_prev)
                old_prev->next = old_modinfo->next;
            else
                modules = old_modinfo->next;
            unregister_module_hooks(old_modinfo->module_id);
            ((void (*)(int)) old_modinfo->stopfunc)(MODSTATE_RELOAD);
            closemodule(old_modinfo->module);
            free_module_functions(old_modinfo);
            free(old_modinfo->name);
            free(old_modinfo);
            break;
        } else
            old_prev = old_modinfo;
    }
    if(!loadModule(libname)) return 0;
    struct ModuleInfo *modinfo;
    for(modinfo = modules; modinfo; modinfo = modinfo->next) {
        if(!(modinfo->state & MODINFO_STATE_STARTED)) {
            modinfo->state |= MODINFO_STATE_STARTED;
            ((void (*)(int)) modinfo->startfunc)(MODSTATE_RELOAD);
        } else
            ((void (*)(int)) modinfo->startfunc)(MODSTATE_REBIND);
    }
    return 1;
}

struct ModuleInfo *ext_get_modules(struct ModuleInfo *last) {
    return (last ? last->next : modules);
}

void start_modules() {
    struct ModuleInfo *modinfo;
    for(modinfo = modules; modinfo; modinfo = modinfo->next) {
        if(!(modinfo->state & MODINFO_STATE_STARTED)) {
            modinfo->state |= MODINFO_STATE_STARTED;
            ((void (*)(int)) modinfo->startfunc)(MODSTATE_STARTSTOP);
        }
    }
}

void stop_modules() {
    struct ModuleInfo *modinfo, *next;
    for(modinfo = modules; modinfo; modinfo = next) {
        next = modinfo->next;
        unregister_module_hooks(modinfo->module_id);
        ((void (*)(int)) modinfo->stopfunc)(MODSTATE_STARTSTOP);
        closemodule(modinfo->module);
        free_module_functions(modinfo);
        free(modinfo->name);
        free(modinfo);
    }
    modules = NULL;
}

static void unregister_module_hooks(int module_id) {
    unregister_module_commands(module_id);
    unregister_module_events(module_id);
    unregister_module_timers(module_id);
    
}

int module_loaded(int module_id) {
    if(!module_id) return 1;
    struct ModuleInfo *modinfo;
    for(modinfo = modules; modinfo; modinfo = modinfo->next) {
        if(modinfo->module_id == module_id)
            return 1;
    }
    return 0;
}

char *get_module_name(int module_id) {
    if(!module_id) return NULL;
    struct ModuleInfo *modinfo;
    for(modinfo = modules; modinfo; modinfo = modinfo->next) {
        if(modinfo->module_id == module_id)
            return modinfo->name;
    }
    return NULL;
}


