/* cmd_neonfun_4stone.c - NeonServ v5.5
 * Copyright (C) 2011-2012  Philipp Kreil (pk910)
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program. If not, see <http://www.gnu.org/licenses/>. 
 */

#include "cmd_neonfun.h"
#include "game_4wins.h"

CMD_BIND(neonfun_cmd_4stone) {
    struct ChanUser *chanuser = getChanUser(user, chan);
    if(!chanuser) return;
    struct fourwins_game *game;
    for(game = fourwins_active_games; game; game = game->next) {
        if(chanuser == game->player[0] || chanuser == game->player[1]) {
            if(game->state == FOURWINS_STATE_WAITING)
                return;
            else 
                break;
        }
    }
    if(game) {
        //check if it's the player's turn
        if(game->player[game->active_player] != chanuser) {
            reply(textclient, user, "NF_4WINS_NOT_YOUR_TURN");
            return;
        }
        int x = atoi(argv[0])-1;
        if(x < 0 || x >= FOURWINS_MATRIX_WIDTH) {
            reply(textclient, user, "NF_4WINS_INVALID_COLUMN");
            return;
        }
        int y = fourwins_next_free_y(game, x);
        if(y < 0) {
            reply(textclient, user, "NF_4WINS_COLUMN_FULL");
            return;
        }
        timeq_del(game->timer);
        game->timer = NULL;
        game->matrix[x][y].field = game->active_player+1;
        fourwins_show_matrix(game);
        if(fourwins_check_win(game, x, y)) {
            fourwins_reply(game, "NF_4WINS_USER_WON", game->player[game->active_player]->user->nick);
            if((game->player[game->active_player]->user->flags & USERFLAG_ISAUTHED)) {
                char *tmp;
                int win_count = ((tmp = getSetting(game->player[game->active_player]->user, chan, "4wins_win")) ? atoi(tmp) : 0);
                int total_win_count = ((tmp = getSetting(game->player[game->active_player]->user, NULL, "4wins_win")) ? atoi(tmp) : 0);
                win_count++;
                total_win_count++;
                char buf[10];
                sprintf(buf, "%d", win_count);
                setSetting(game->player[game->active_player]->user, chan, "4wins_win", buf);
                sprintf(buf, "%d", total_win_count);
                setSetting(game->player[game->active_player]->user, NULL, "4wins_win", buf);
            }
            fourwins_free_game(game);
        } else {
            game->total_stones++;
            if(game->total_stones == FOURWINS_MAX_STONES) {
                fourwins_reply(game, "NF_4WINS_DRAW");
                if((game->player[0]->user->flags & USERFLAG_ISAUTHED)) {
                    char *tmp;
                    int win_count = ((tmp = getSetting(game->player[0]->user, chan, "4wins_draw")) ? atoi(tmp) : 0);
                    int total_win_count = ((tmp = getSetting(game->player[0]->user, NULL, "4wins_draw")) ? atoi(tmp) : 0);
                    win_count++;
                    total_win_count++;
                    char buf[10];
                    sprintf(buf, "%d", win_count);
                    setSetting(game->player[0]->user, chan, "4wins_draw", buf);
                    sprintf(buf, "%d", total_win_count);
                    setSetting(game->player[0]->user, NULL, "4wins_draw", buf);
                }
                if((game->player[1]->user->flags & USERFLAG_ISAUTHED)) {
                    char *tmp;
                    int win_count = ((tmp = getSetting(game->player[1]->user, chan, "4wins_draw")) ? atoi(tmp) : 0);
                    int total_win_count = ((tmp = getSetting(game->player[1]->user, NULL, "4wins_draw")) ? atoi(tmp) : 0);
                    win_count++;
                    total_win_count++;
                    char buf[10];
                    sprintf(buf, "%d", win_count);
                    setSetting(game->player[1]->user, chan, "4wins_draw", buf);
                    sprintf(buf, "%d", total_win_count);
                    setSetting(game->player[1]->user, NULL, "4wins_draw", buf);
                }
                fourwins_free_game(game);
                return;
            }
            if(game->active_player)
                game->active_player = 0;
            else
                game->active_player = 1;
            fourwins_reply(game, "NF_4WINS_USER_HURRY_UP", game->player[game->active_player]->user->nick);
            game->timer = timeq_add(120, module_id, fourwins_timeout, game);
        }
    }
}
