/* cmd_neonserv_halfop.c - NeonServ v5.5
 * Copyright (C) 2011-2012  Philipp Kreil (pk910)
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program. If not, see <http://www.gnu.org/licenses/>. 
 */

#include "cmd_neonserv.h"

/*
* argv[0-*]    nicks
*/
static USERLIST_CALLBACK(neonserv_cmd_halfop_userlist_lookup);
static void neonserv_cmd_halfop_async1(struct ClientSocket *client, struct ClientSocket *textclient, struct UserNode *user, struct ChanNode *chan, struct Event *event, char *nicks);

struct neonserv_cmd_halfop_cache {
    struct ClientSocket *client, *textclient;
    struct UserNode *user;
    struct Event *event;
    char *nicks;
};

CMD_BIND(neonserv_cmd_halfop) {
    struct neonserv_cmd_halfop_cache *cache = malloc(sizeof(*cache));
    if (!cache) {
        perror("malloc() failed");
        return;
    }
    cache->client = client;
    cache->textclient = textclient;
    cache->user = user;
    cache->event = event;
    cache->nicks = strdup(merge_argv(argv, 0, argc));
    get_userlist_if_invisible(chan, module_id, neonserv_cmd_halfop_userlist_lookup, cache);
}

static USERLIST_CALLBACK(neonserv_cmd_halfop_userlist_lookup) {
    struct neonserv_cmd_halfop_cache *cache = data;
    neonserv_cmd_halfop_async1(cache->client, cache->textclient, cache->user, chan, cache->event, cache->nicks);
    free(cache->nicks);
    free(cache);
}

static void neonserv_cmd_halfop_async1(struct ClientSocket *client, struct ClientSocket *textclient, struct UserNode *user, struct ChanNode *chan, struct Event *event, char *nicks) {
    int total_users = 0, done_users = 0;
    struct UserNode *cuser;
    struct ChanUser *chanuser;
    struct ModeBuffer *modeBuf;
    modeBuf = initModeBuffer(client, chan);
    char *a, *b = nicks;
    do {
        a = strstr(b, " ");
        if(a) *a = '\0';
        total_users++;
        cuser = searchUserByNick(b);
        if(!cuser) {
            //check for an invisible user
            for(chanuser = getChannelUsers(chan, NULL); chanuser; chanuser = getChannelUsers(chan, chanuser)) {
                if(!stricmp(chanuser->user->nick, b)) {
                    cuser = chanuser->user;
                    break;
                }
            }
            if(!cuser) continue;
        } else {
            chanuser = getChanUser(cuser, chan);
            if(!chanuser) continue;
        }
        done_users++;
        if(chanuser->flags & CHANUSERFLAG_OPPED) continue;
        modeBufferHalfop(modeBuf, b);
        if(a) {
            b = a+1;
        }
    } while(a);
    freeModeBuffer(modeBuf);
    if(done_users == total_users)
        reply(textclient, user, "NS_HALFOP_DONE", chan->name);
    else
        reply(textclient, user, "NS_HALFOP_FAIL", client->user->nick);
    if(done_users) 
        logEvent(event);
}
