/* cmd_neonserv_setrank.c - NeonServ v5.5
 * Copyright (C) 2011-2012  Philipp Kreil (pk910)
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program. If not, see <http://www.gnu.org/licenses/>. 
 */

#include "cmd_neonserv.h"

/*
* argv[0]   rank id
* argv[1]   setting
* argv[2-*] value
*/

static void neonserv_cmd_setrank_name(struct ClientSocket *textclient, struct UserNode *user, int rank_id, char *current, char *value);
static void neonserv_cmd_setrank_info(struct ClientSocket *textclient, struct UserNode *user, int rank_id, char *current, char *value);
static void neonserv_cmd_setrank_access(struct ClientSocket *textclient, struct UserNode *user, int rank_id, char *current, char *value);
static void neonserv_cmd_setrank_order(struct ClientSocket *textclient, struct UserNode *user, int rank_id, char *current, char *value);

CMD_BIND(neonserv_cmd_setrank) {
    MYSQL_RES *res;
    MYSQL_ROW row;
    printf_mysql_query("SELECT `rank_id`, `rank_name`, `rank_info`, `rank_access`, `rank_order` FROM `support_ranks` WHERE `rank_id` = '%s'", escape_string(argv[0]));
    res = mysql_use();
    if ((row = mysql_fetch_row(res)) == NULL) {
        reply(textclient, user, "NS_SETRANK_NOT_FOUND", argv[0]);
        return;
    }
    if(argc < 2) {
        reply(textclient, user, "NS_SETRANK_HEAD", row[0]);
        reply(textclient, user, "\002NAME   \002 %s", row[1]);
        reply(textclient, user, "\002INFO   \002 %s", row[2]);
        reply(textclient, user, "\002ACCESS \002 %s", row[3]);
        reply(textclient, user, "\002ORDER  \002 %s", row[4]);
    } else {
        char *value = NULL;
        if(argc > 2) {
            value = merge_argv(argv, 2, argc);
        }
        if(!stricmp(argv[1], "name"))
            neonserv_cmd_setrank_name(textclient, user, atoi(row[0]), row[1], value);
        else if(!stricmp(argv[1], "info"))
            neonserv_cmd_setrank_info(textclient, user, atoi(row[0]), row[2], value);
        else if(!stricmp(argv[1], "access"))
            neonserv_cmd_setrank_access(textclient, user, atoi(row[0]), row[3], value);
        else if(!stricmp(argv[1], "order"))
            neonserv_cmd_setrank_order(textclient, user, atoi(row[0]), row[4], value);
        else
            reply(textclient, user, "NS_SETRANK_UNKNOWN_SETTING", row[1]);
    }
}

static void neonserv_cmd_setrank_name(struct ClientSocket *textclient, struct UserNode *user, int rank_id, char *current, char *value) {
    if(value && stricmp(value, current)) {
        MYSQL_RES *res;
        MYSQL_ROW row;
        printf_mysql_query("SELECT `rank_id` FROM `support_ranks` WHERE `rank_name` = '%s'", escape_string(value));
        res = mysql_use();
        if ((row = mysql_fetch_row(res)) != NULL) {
            reply(textclient, user, "NS_ADDRANK_EXISTS", value);
            return;
        }
        printf_mysql_query("UPDATE `support_ranks` SET `rank_name` = '%s' WHERE `rank_id` = '%d'", escape_string(value), rank_id);
        current = value;
    }
    reply(textclient, user, "\002NAME\002 %s", current);
}

static void neonserv_cmd_setrank_info(struct ClientSocket *textclient, struct UserNode *user, int rank_id, char *current, char *value) {
    if(value && stricmp(value, current)) {
        printf_mysql_query("UPDATE `support_ranks` SET `rank_info` = '%s' WHERE `rank_id` = '%d'", escape_string(value), rank_id);
        current = value;
    }
    reply(textclient, user, "\002INFO\002 %s", current);
}

static void neonserv_cmd_setrank_access(struct ClientSocket *textclient, struct UserNode *user, int rank_id, char *current, char *value) {
    if(value && stricmp(value, current)) {
        int new_access = atoi(value);
        if(new_access <= 0 || new_access > 1000) {
            reply(textclient, user, "NS_INVALID_ACCESS", new_access);
            return;
        }
        printf_mysql_query("UPDATE `users` SET `user_access` = '%d' WHERE `user_rank` = '%d'", new_access, rank_id);
        printf_mysql_query("UPDATE `support_ranks` SET `rank_access` = '%d' WHERE `rank_id` = '%d'", new_access, rank_id);
        current = value;
    }
    reply(textclient, user, "\002ACCESS\002 %s", current);
}

static void neonserv_cmd_setrank_order(struct ClientSocket *textclient, struct UserNode *user, int rank_id, char *current, char *value) {
    if(value && stricmp(value, current)) {
        int new_order = atoi(value);
        if(new_order <= 0 || new_order > 99) {
            reply(textclient, user, "NS_SETRANK_ORDER_INVALID", new_order);
            return;
        }
        printf_mysql_query("UPDATE `support_ranks` SET `rank_order` = '%d' WHERE `rank_id` = '%d'", new_order, rank_id);
        current = value;
    }
    reply(textclient, user, "\002ORDER\002 %s", current);
}
