/* cmd_neonserv_trim.c - NeonServ v5.5
 * Copyright (C) 2011-2012  Philipp Kreil (pk910)
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program. If not, see <http://www.gnu.org/licenses/>. 
 */

#include "cmd_neonserv.h"

/*
* argv[0]  target (format: minaccess-maxaccess/users/bans)
* argv[1]  duration
*/
static USERLIST_CALLBACK(neonserv_cmd_trim_userlist_lookup);
static void neonserv_cmd_trim_async1(struct ClientSocket *client, struct ClientSocket *textclient, struct UserNode *user, struct ChanNode *chan, struct Event *event, int min_access, int max_access, int duration);

struct neonserv_cmd_trim_cache {
    struct ClientSocket *client, *textclient;
    struct UserNode *user;
    struct Event *event;
    int min_access;
    int max_access;
    int duration;
};

CMD_BIND(neonserv_cmd_trim) {
    if(stricmp(argv[0], "bans") && !checkChannelAccess(user, chan, "channel_candel", 0)) {
        if(isGodMode(user)) {
            event->flags |= CMDFLAG_OPLOG;
        } else {
            reply(textclient, user, "NS_ACCESS_DENIED");
            return;
        }
    }
    int min_access, max_access;
    int duration = strToTime(user, argv[1]);
    if(duration < 30) {
        reply(textclient, user, "NS_TRIM_DURATION_TOO_SHORT", 30);
        return;
    }
    if(!stricmp(argv[0], "users")) {
        min_access = 1;
        max_access = getChannelAccess(user, chan) - 1;
    } else if(!stricmp(argv[0], "bans")) {
        if(!checkChannelAccess(user, chan, "channel_staticban", 0)) {
            if(isGodMode(user)) {
                event->flags |= CMDFLAG_OPLOG;
            } else {
                reply(textclient, user, "NS_ACCESS_DENIED");
                return;
            }
        }
        MYSQL_RES *res;
        MYSQL_ROW row;
        char nameBuf[20];
        printf_mysql_query("SELECT `ban_mask`, `ban_id`, `ban_timeout` FROM `bans` WHERE `ban_channel` = '%d' AND `ban_triggered` < %d", chan->channel_id, (int) (time(0) - duration));
        res = mysql_use();
        int bancount = mysql_num_rows(res);
        struct ModeBuffer *modenode = initModeBuffer(client, chan);
        while ((row = mysql_fetch_row(res)) != NULL) {
            if(strcmp(row[2], "0")) {
                sprintf(nameBuf, "ban_%s", row[1]);
                timeq_del_name(nameBuf);
            }
            printf_mysql_query("DELETE FROM `bans` WHERE `ban_id` = '%s'", row[1]);
            modeBufferUnban(modenode, row[0]);
        }
        freeModeBuffer(modenode);
        char timeBuf[MAXLEN];
        reply(textclient, user, "NS_TRIM_BAN_DONE", bancount, chan->name, timeToStr(user, duration, 3, timeBuf));
        if(bancount)
            logEvent(event);
        return;
    } else {
        char *seperator = strstr(argv[0], "-");
        if(seperator) {
            *seperator = '\0';
            seperator++;
            min_access = atoi(argv[0]);
            max_access = atoi(seperator);
            if(max_access < min_access) {
                reply(textclient, user, "NS_INVALID_ACCESS_RANGE", min_access, max_access);
                return;
            }
        } else {
            min_access = atoi(argv[0]);
            max_access = min_access;
        }
        if(max_access >= getChannelAccess(user, chan)) {
            if(isGodMode(user)) {
                event->flags |= CMDFLAG_OPLOG;
            } else {
                reply(textclient, user, "NS_NO_ACCESS");
                return;
            }
        }
    }
    struct neonserv_cmd_trim_cache *cache = malloc(sizeof(*cache));
    if (!cache) {
        perror("malloc() failed");
        return;
    }
    cache->client = client;
    cache->textclient = textclient;
    cache->user = user;
    cache->event = event;
    cache->min_access = min_access;
    cache->max_access = max_access;
    cache->duration = duration;
    get_userlist_with_invisible(chan, module_id, neonserv_cmd_trim_userlist_lookup, cache);
}

static USERLIST_CALLBACK(neonserv_cmd_trim_userlist_lookup) {
    struct neonserv_cmd_trim_cache *cache = data;
    //got userlist
    neonserv_cmd_trim_async1(cache->client, cache->textclient, cache->user, chan, cache->event, cache->min_access, cache->max_access, cache->duration);
    free(cache);
}

static void neonserv_cmd_trim_async1(struct ClientSocket *client, struct ClientSocket *textclient, struct UserNode *user, struct ChanNode *chan, struct Event *event, int min_access, int max_access, int duration) {
    MYSQL_RES *res;
    MYSQL_ROW row;
    int trim_count = 0, is_here;
    struct ChanUser *chanuser;
    printf_mysql_query("SELECT `chanuser_seen`, `user_user`, `chanuser_id` FROM `chanusers` LEFT JOIN `users` ON `chanuser_uid` = `user_id` WHERE `chanuser_cid` = '%d' AND `chanuser_access` >= '%d' AND `chanuser_access` <= '%d'", chan->channel_id, min_access, max_access);
    res = mysql_use();
    while ((row = mysql_fetch_row(res)) != NULL) {
        if(!strcmp(row[0], "0") || time(0) - atoi(row[0]) >= duration) {
            //check if the user is currently in the channel
            is_here = 0;
            for(chanuser = getChannelUsers(chan, NULL); chanuser; chanuser = getChannelUsers(chan, chanuser)) {
                if((chanuser->user->flags & USERFLAG_ISAUTHED) && !strcmp(chanuser->user->auth, row[1])) {
                    is_here = 1;
                    break;
                }
            }
            if(!is_here) {
                //delete the user
                trim_count++;
                printf_mysql_query("DELETE FROM `chanusers` WHERE `chanuser_id` = '%s'", row[2]);
            }
        }
    }
    char timeBuf[MAXLEN];
    reply(textclient, user, "NS_TRIM_DONE", trim_count, min_access, max_access, chan->name, timeToStr(user, duration, 3, timeBuf));
    if(trim_count)
        logEvent(event);
}
