/* bot_NeonSpam.c - NeonServ v5.5
 * Copyright (C) 2011-2012  Philipp Kreil (pk910)
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program. If not, see <http://www.gnu.org/licenses/>. 
 */
#include "../module.h"
#include "../botid.h"

#include "bot_NeonSpam.h"
#include "../../modcmd.h"
#include "../../IRCParser.h"
#include "../../IRCEvents.h"
#include "../../UserNode.h"
#include "../../ChanNode.h"
#include "../../ChanUser.h"
#include "../../ModeNode.h"
#include "../../BanNode.h"
#include "../../ClientSocket.h"
#include "../../mysqlConn.h"
#include "../../lang.h"
#include "../../HandleInfoHandler.h"
#include "../../WHOHandler.h"
#include "../../DBHelper.h"
#include "../../tools.h"
#include "../../timeq.h"
#include "../../version.h"
#include "../../EventLogger.h"
#include "../../bots.h"
#include "cmd_neonspam.h"

#define BOTID NEONSPAM_BOTID
#define BOTALIAS "NeonSpam"

static const struct default_language_entry msgtab[] = {
    {"SS_SET_PERCENT", "%u is an invalid percent value (valid: 0-100)"}, /* {ARGS: 120} */
    {"SS_SET_SENSIBILITY", "%s is an invalid sensibility format. (valid: amount:time   e.g. 5:10)"}, /* {ARGS: "mp"} */
    {"SS_SET_SENSIBILITY_AMOUNT", "%d is an invalid amount value. (valid: %d-%d)"}, /* {ARGS: 120, 1, 5} */
    {"SS_SET_SENSIBILITY_TIME", "%d is an invalid time value. (valid: %d-%d)"}, /* {ARGS: 300, 15, 180} */
    {"SS_SET_SPAMLIMIT", "%d is an invalid spamlimit value. (valid: %d-%d)"}, /* {ARGS: 120, 2, 8} */
    {"SS_SET_OPTION_SpamReaction_0", "Kick"},
    {"SS_SET_OPTION_SpamReaction_1", "KickBan"},
    {"SS_SET_OPTION_SpamReaction_2", "Timed Ban"},
    {"SS_SET_OPTION_FloodReaction_0", "Kick"},
    {"SS_SET_OPTION_FloodReaction_1", "KickBan"},
    {"SS_SET_OPTION_FloodReaction_2", "Timed Ban"},
    {"SS_SET_OPTION_JoinReaction_0", "Kick"},
    {"SS_SET_OPTION_JoinReaction_1", "KickBan"},
    {"SS_SET_OPTION_JoinReaction_2", "Timed Ban"},
    {"SS_SET_OPTION_CapsReaction_0", "Kick"},
    {"SS_SET_OPTION_CapsReaction_1", "KickBan"},
    {"SS_SET_OPTION_CapsReaction_2", "Timed Ban"},
    {"SS_SET_OPTION_DigitReaction_0", "Kick"},
    {"SS_SET_OPTION_DigitReaction_1", "KickBan"},
    {"SS_SET_OPTION_DigitReaction_2", "Timed Ban"},
    {"SS_SET_OPTION_BadwordReaction_0", "Kick"},
    {"SS_SET_OPTION_BadwordReaction_1", "KickBan"},
    {"SS_SET_OPTION_BadwordReaction_2", "Timed Ban"},
    {"SS_BADWORD_ADDED", "Badword '%s' added. (ID: %d)"}, /* {ARGS: "NeonServ is bad", 1} */
    {"SS_BADWORD_ID_UNKNOWN", "Badword ID %d was not found in %s."}, /* {ARGS: 1, "#channel"} */
    {"SS_BADWORD_DELETED", "Badword '%s' deleted. (ID: %d)"}, /* {ARGS: "NeonServ is bad", 1} */
    {"SS_BADWORDS_REACTION_0", "Kick"},
    {"SS_BADWORDS_REACTION_1", "KickBan"},
    {"SS_BADWORDS_REACTION_2", "Timed Ban"},
    {"SS_BADWORDS_ID", "Id"},
    {"SS_BADWORDS_PATTERN", "Pattern"},
    {"SS_BADWORDS_SCAN", "Scan"},
    {"SS_BADWORDS_REACTION", "Reaction"},
    {NULL, NULL}
};

static unsigned int convertNeonSpamSettingsToFlags(char *str);
static void createSpamNode(struct ChanUser *chanuser);
static void freeJoinNode(struct NeonSpamJoinNode *joinnode);
static struct NeonSpamJoinNode *getNeonSpamJoinNode(struct ChanUser *chanuser);

#define SPAMSERV_CHECK_IGNORE 0
#define SPAMSERV_CHECK_WARN   1
#define SPAMSERV_CHECK_PUNISH 2
#define SPAMSERV_CHECK_DEAD   3 /* scanner has already killed the user */

#define SPAMSERV_MSG_SPAM       "Spamming"
#define SPAMSERV_MSG_FLOOD      "Flooding the channel/network"
#define SPAMSERV_MSG_ADV        "Advertising"
#define SPAMSERV_MSG_JOINFLOOD  "Join flooding the channel"
#define SPAMSERV_MSG_WARNING    "%s is against the channel rules"
#define SPAMSERV_MSG_BOTNET     "BotNet detected."
#define SPAMSERV_MSG_CAPS       "Using too many chars in UPPER CASE"
#define SPAMSERV_MSG_DIGIT      "Using too many numeric chars"
#define SPAMSERV_MSG_BADWORD    "Your message contained a forbidden word."

//EVENTS
#include "event_neonspam_join.c"
#include "event_neonspam_chanmsg.c"

static void neonspam_bot_ready(struct ClientSocket *client) {
    if(client->botid != BOTID)
        return;
    MYSQL_RES *res;
    MYSQL_ROW row;
    
    printf_mysql_query("SELECT `automodes`, `oper_user`, `oper_pass` FROM `bots` WHERE `id` = '%d'", client->clientid);
    res = mysql_use();
    if ((row = mysql_fetch_row(res)) != NULL) {
        if(row[1] && row[2]) {
            putsock(client, "OPER %s %s", row[1], row[2]);
        }
        putsock(client, "MODE %s +%s", client->user->nick, row[0]);
    }
    
    printf_mysql_query("SELECT `channel_name`, `channel_key` FROM `bot_channels` LEFT JOIN `channels` ON `chanid` = `channel_id` WHERE `botid` = '%d' AND `suspended` = '0'", client->clientid);
    res = mysql_use();
    
    while ((row = mysql_fetch_row(res)) != NULL) {
        putsock(client, "JOIN %s %s", row[0], row[1]);
    }
}

static void neonspam_trigger_callback(int clientid, struct ChanNode *chan, char *trigger) {
    MYSQL_RES *res;
    MYSQL_ROW row;
    loadChannelSettings(chan);
    if(!(chan->flags & CHANFLAG_CHAN_REGISTERED)) {
        strcpy(trigger, "~");
        return;
    }
    printf_mysql_query("SELECT `trigger`, `defaulttrigger` FROM `bot_channels` LEFT JOIN `bots` ON `botid` = `bots`.`id` WHERE `chanid` = '%d' AND `botclass` = '%d'", chan->channel_id, BOTID);
    res = mysql_use();
    if(!(row = mysql_fetch_row(res))) {
        strcpy(trigger, "~");
        return;
    }
    if(row[0] && *row[0])
        strcpy(trigger, row[0]);
    else
        strcpy(trigger, ((row[1] && *row[1]) ? row[1] : "~"));
}

static void start_bots(int type) {
    struct ClientSocket *client;
    MYSQL_RES *res, *res2;
    MYSQL_ROW row;
    
    if(type == MODSTATE_STARTSTOP) {
        printf_mysql_query("SELECT `nick`, `ident`, `realname`, `server`, `port`, `pass`, `textbot`, `id`, `queue`, `ssl`, `bind`, `secret` FROM `bots` WHERE `botclass` = '%d' AND `active` = '1'", BOTID);
        res = mysql_use();
        
        while ((row = mysql_fetch_row(res)) != NULL) {
            client = create_socket(row[3], atoi(row[4]), row[10], row[5], row[0], row[1], row[2]);
            client->flags |= (strcmp(row[6], "0") ? SOCKET_FLAG_PREFERRED : 0);
            client->flags |= (strcmp(row[8], "0") ? SOCKET_FLAG_USE_QUEUE : 0);
            client->flags |= (strcmp(row[9], "0") ? SOCKET_FLAG_SSL : 0);
            client->flags |= (strcmp(row[11], "0") ? SOCKET_FLAG_SECRET_BOT : 0);
            client->flags |= SOCKET_FLAG_REQUEST_INVITE | SOCKET_FLAG_REQUEST_OP;
            client->botid = BOTID;
            client->clientid = atoi(row[7]);
            connect_socket(client);
        }
    }
    
    printf_mysql_query("SELECT `command`, `function`, `parameters`, `global_access`, `chan_access`, `flags` FROM `bot_binds` WHERE `botclass` = '%d'", BOTID);
    res2 = mysql_use();
    while ((row = mysql_fetch_row(res2)) != NULL) {
        if(bind_cmd_to_command(BOTID, row[0], row[1])) {
            if(row[2] && strcmp(row[2], "")) {
                bind_set_parameters(BOTID, row[0], row[2]);
            }
            if(row[3]) {
                bind_set_global_access(BOTID, row[0], atoi(row[3]));
            }
            if(row[4]) {
                bind_set_channel_access(BOTID, row[0], row[4]);
            }
            if(strcmp(row[5], "0"))
                bind_set_bind_flags(BOTID, row[0], atoi(row[5]));
        }
    }
    bind_unbound_required_functions(BOTID);
}

char* convertNeonSpamSettingsToString(unsigned int flags, char *buffer) {
    int pos = 0;
    unsigned int i;
    int j = 0;
    char *chars = SPAMSETTINGS_CHARS;
    for(i = 1; i <= SPAMSETTINGS_FLAGS; i = i << 1) {
        if(flags & i)
            buffer[pos++] = chars[j];
        j++;
    }
    buffer[pos] = '\0';
    return buffer;
}

static unsigned int convertNeonSpamSettingsToFlags(char *str) {
    unsigned int i = 1, flags = 0;
    int j = 0;
    char *chars = SPAMSETTINGS_CHARS;
    while(*str) {
        for(; i <= SPAMSETTINGS_FLAGS; i = i << 1) {
            if(*str == chars[j]) {
                flags |= i;
                j++;
                i = i << 1;
                break;
            }
            j++;
        }
        str++;
    }
    return flags;
}

int loadNeonSpamSettings(struct ChanNode *chan) {
    if(chan->spam_settings) return 0;
    struct NeonSpamSettings *settings = malloc(sizeof(*settings));
    if(!settings) {
        perror("malloc() failed");
        return 0;
    }
    MYSQL_RES *res;
    MYSQL_ROW row, defaults = NULL;
    loadChannelSettings(chan);
    printf_mysql_query("SELECT `channel_scanner`, `channel_spam_limit`, `channel_spam_except`, `channel_flood_limit`, `channel_flood_time`, `channel_flood_except`, `channel_join_limit`, `channel_join_time`, `channel_join_except`, `channel_caps_percent`, `channel_caps_except`, `channel_digit_percent`, `channel_digit_except`, `channel_badword_except` FROM `channels` WHERE `channel_id` = '%d'", chan->channel_id);
    res = mysql_use();
    if(!(row = mysql_fetch_row(res)))
        return 0;
    if(!row[0] || !row[1] || !row[2] || !row[3] || !row[4] || !row[5] || !row[6] || !row[7] || !row[8] || !row[9] || !row[10] || !row[11] || !row[12]) {
        printf_mysql_query("SELECT `channel_scanner`, `channel_spam_limit`, `channel_spam_except`, `channel_flood_limit`, `channel_flood_time`, `channel_flood_except`, `channel_join_limit`, `channel_join_time`, `channel_join_except`, `channel_caps_percent`, `channel_caps_except`, `channel_digit_percent`, `channel_digit_except`, `channel_badword_except` FROM `channels` WHERE `channel_name` = 'defaults'");
        res = mysql_use();
        defaults = mysql_fetch_row(res);
    }
    settings->flags = convertNeonSpamSettingsToFlags(row[0] ? row[0] : defaults[0]);
    settings->spam_amount = atoi(row[1] ? row[1] : defaults[1]);
    settings->exceptlevel[SPAMSETTINGS_SPAMEXCINDEX] = atoi(row[2] ? row[2] : defaults[2]);
    settings->sensibility_amount[SPAMSETTINGS_FLOODSENINDEX] = atoi(row[3] ? row[3] : defaults[3]);
    settings->sensibility_time[SPAMSETTINGS_FLOODSENINDEX] = atoi(row[4] ? row[4] : defaults[4]);
    settings->exceptlevel[SPAMSETTINGS_FLOODEXCINDEX] = atoi(row[5] ? row[5] : defaults[5]);
    settings->sensibility_amount[SPAMSETTINGS_JOINSENINDEX] = atoi(row[6] ? row[6] : defaults[6]);
    settings->sensibility_time[SPAMSETTINGS_JOINSENINDEX] = atoi(row[7] ? row[7] : defaults[7]);
    settings->exceptlevel[SPAMSETTINGS_JOINEXCINDEX] = atoi(row[8] ? row[8] : defaults[8]);
    settings->percent[SPAMSETTINGS_CAPSPERCENTINDEX] = atoi(row[9] ? row[9] : defaults[9]);
    settings->exceptlevel[SPAMSETTINGS_CAPSEXCINDEX] = atoi(row[10] ? row[10] : defaults[10]);
    settings->percent[SPAMSETTINGS_DIGITPERCENTINDEX] = atoi(row[11] ? row[11] : defaults[11]);
    settings->exceptlevel[SPAMSETTINGS_DIGITEXCINDEX] = atoi(row[12] ? row[12] : defaults[12]);
    settings->exceptlevel[SPAMSETTINGS_BADWORDEXCINDEX] = atoi(row[13] ? row[13] : defaults[13]);
    settings->join_nodes = NULL;
    settings->lastmsg_time = 0;
    int i;
    for(i = 0; i < BOTNETSCAN_USERS; i++) 
        settings->botnicks[i] = NULL;
    settings->badwords = NULL;
    printf_mysql_query("SELECT `badword_match`, `badword_reaction`, `badword_reaction_time`, `badword_id`, `badword_scan_ops`, `badword_scan_voice`, `badword_exceptlevel`, `badword_use_default`, `badword_use_default_reaction` FROM `spamserv_badwords` WHERE `badword_cid` = '%d'", chan->channel_id);
    res = mysql_use();
    while((row = mysql_fetch_row(res))) {
        struct NeonSpamBadword *badword = malloc(sizeof(*badword));
        badword->id = atoi(row[3]);
        badword->badword = strdup(row[0]);
        badword->use_defaults = atoi(row[7]);
        badword->exceptlevel = atoi(row[6]);
        badword->scan_ops = atoi(row[4]);
        badword->scan_voice = atoi(row[5]);
        badword->use_default_reaction = atoi(row[8]);
        badword->reaction = atoi(row[1]);
        badword->reaction_time = atoi(row[2]);
        badword->next = settings->badwords;
        settings->badwords = badword;
    }
    chan->spam_settings = settings;
    return 1;
}

void freeNeonSpamSettings(struct NeonSpamSettings *settings) {
    struct NeonSpamJoinNode *joinnode, *nextjoinnode;
    for(joinnode = settings->join_nodes; joinnode; joinnode = nextjoinnode) {
        nextjoinnode = joinnode->next;
        freeJoinNode(joinnode);
    }
    struct NeonSpamBadword *badword, *nextbadword;
    for(badword = settings->badwords; badword; badword = nextbadword) {
        nextbadword = badword->next;
        free(badword->badword);
        free(badword);
    }
    free(settings);
}

static void freeJoinNode(struct NeonSpamJoinNode *joinnode) {
    free(joinnode->ident);
    free(joinnode->host);
    free(joinnode);
}

static struct NeonSpamJoinNode *getNeonSpamJoinNode(struct ChanUser *chanuser) {
    struct NeonSpamJoinNode *joinnode, *prevjoinnode = NULL, *nextjoinnode, *result = NULL;
    for(joinnode = chanuser->chan->spam_settings->join_nodes; joinnode; joinnode = nextjoinnode) {
        nextjoinnode = joinnode->next;
        if(!stricmp(joinnode->ident, chanuser->user->ident) && !stricmp(joinnode->host, chanuser->user->host)) {
            prevjoinnode = joinnode;
            result = joinnode;
        } else if(time(0) - joinnode->last_penalty_update > MAX_JOIN_TIME) {
            freeJoinNode(joinnode);
            if(prevjoinnode)
                prevjoinnode->next = nextjoinnode;
            else
                chanuser->chan->spam_settings->join_nodes = nextjoinnode;
        } else 
            prevjoinnode = joinnode;
    }
    if(result)
        return result;
    joinnode = malloc(sizeof(*joinnode));
    if(!joinnode) {
        perror("malloc() failed");
        return NULL;
    }
    joinnode->ident = strdup(chanuser->user->ident);
    joinnode->host = strdup(chanuser->user->host);
    joinnode->last_penalty_update = time(0);
    joinnode->joinpenalty = 0;
    joinnode->next = chanuser->chan->spam_settings->join_nodes;
    chanuser->chan->spam_settings->join_nodes = joinnode;
    return joinnode;
}

static void createSpamNode(struct ChanUser *chanuser) {
    struct NeonSpamNode *spamnode = malloc(sizeof(*spamnode));
    if(!spamnode) {
        perror("malloc() failed");
        return;
    }
    spamnode->lastmsg = 0;
    spamnode->spamcount = 0;
    spamnode->floodpenalty = 0;
    spamnode->last_penalty_update = time(0);
    chanuser->spamnode = spamnode;
}

static int neonspam_event_freechan(struct ChanNode *chan) {
    if(chan->spam_settings)
        freeNeonSpamSettings(chan->spam_settings);
    return 1;
}

static void neonspam_event_invite(struct ClientSocket *client, struct UserNode *user, char *channel) {
	if(client->botid != BOTID)
		return;
    MYSQL_RES *res;
    MYSQL_ROW row;
    printf_mysql_query("SELECT `botid`, `bot_channels`.`id`, `suspended` FROM `bot_channels` LEFT JOIN `bots` ON `bot_channels`.`botid` = `bots`.`id` LEFT JOIN `channels` ON `chanid` = `channel_id` WHERE `channel_name` = '%s' AND `botclass` = '%d'", escape_string(channel), client->botid);
    res = mysql_use();
    if ((row = mysql_fetch_row(res)) == NULL) {
        reply(client, user, "NS_INVITE_FAIL", channel, client->user->nick);
        return;
    }
    if(!strcmp(row[2], "1")) {
        reply(client, user, "MODCMD_CHAN_SUSPENDED");
        return;
    }
    int botid = atoi(row[0]);
    struct ClientSocket *bot;
    for(bot = getBots(SOCKET_FLAG_READY, NULL); bot; bot = getBots(SOCKET_FLAG_READY, bot)) {
        if(bot->clientid == botid)
            break;
    }
    if(bot) {
        struct ChanNode *chan = getChanByName(channel);
        if(chan && isUserOnChan(bot->user, chan)) {
            reply(client, user, "NS_INVITE_ON_CHAN", bot->user->nick, chan->name);
        } else
            putsock(bot, "JOIN %s", channel);
    }
}

void init_NeonSpam(int type) {
    
    set_bot_alias(BOTID, BOTALIAS);
    start_bots(type);
    
    if(type == MODSTATE_REBIND) return;
    
    //register events
    bind_bot_ready(neonspam_bot_ready, module_id);
    bind_join(neonspam_event_join, module_id);
    bind_chanmsg(neonspam_event_chanmsg, module_id);
    bind_privctcp(general_event_privctcp, module_id);
    bind_freechan(neonspam_event_freechan, module_id);
    bind_invite(neonspam_event_invite, module_id);
	
    set_trigger_callback(BOTID, module_id, neonspam_trigger_callback);
    
    register_default_language_table(msgtab);
}

void free_NeonSpam(int type) {
    unbind_allcmd(BOTID);
    if(type == MODSTATE_STARTSTOP) {
        //disconnect all our bots
        struct ClientSocket *client;
        for(client = getBots(0, NULL); client; client = getBots(0, client)) {
            if(client->botid == BOTID) {
                unbind_botwise_allcmd(0, client->clientid);
                close_socket(client);
                break;
            }
        }
    }
}

#undef BOTID
#undef BOTALIAS
