/* tools.h - NeonServ v5.5
 * Copyright (C) 2011-2012  Philipp Kreil (pk910)
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program. If not, see <http://www.gnu.org/licenses/>. 
 */
#ifndef _tools_h
#define _tools_h

#include "main.h"

#define TABLE_FLAG_USE_POINTER   0x01
#define TABLE_FLAG_COL_BOLD      0x02
#define TABLE_FLAG_COL_CONTENTS  0x04
#define TABLE_FLAG_COL_SKIP_NULL 0x08

struct ClientSocket;
struct UserNode;
struct ChanNode;

struct Table {
    char ***contents;
    int length;
    int width;
    int flags;
    int *col_flags;
    
    int entrys;
    int *maxwidth;
    
    char **table_lines; //we just store this to free it in table_free
};

struct ModeBuffer {
    struct ClientSocket *client;
    struct ChanNode *chan;
    char addModes[MAXMODES];
    char delModes[MAXMODES];
    char *addModesParams[MAXMODES];
    char *delModesParams[MAXMODES];
    int addCount;
    int delCount;
};

#define modeBufferSimpleMode(MODEBUF,ADD,MODE) modeBufferSet(MODEBUF, ADD, MODE, NULL) 
#define modeBufferOp(MODEBUF,USER) modeBufferSet(MODEBUF, 1, 'o', USER)
#define modeBufferDeop(MODEBUF,USER) modeBufferSet(MODEBUF, 0, 'o', USER) 
#define modeBufferHalfop(MODEBUF,USER) modeBufferSet(MODEBUF, 1, 'h', USER)
#define modeBufferDehalfop(MODEBUF,USER) modeBufferSet(MODEBUF, 0, 'h', USER) 
#define modeBufferVoice(MODEBUF,USER) modeBufferSet(MODEBUF, 1, 'v', USER)
#define modeBufferDevoice(MODEBUF,USER) modeBufferSet(MODEBUF, 0, 'v', USER) 
#define modeBufferBan(MODEBUF,MASK) modeBufferSet(MODEBUF, 1, 'b', MASK) 
#define modeBufferUnban(MODEBUF,MASK) modeBufferSet(MODEBUF, 0, 'b', MASK) 

#ifndef DND_FUNCTIONS
/* MODULAR ACCESSIBLE */ int match(const char *mask, const char *name);

/* MODULAR ACCESSIBLE */ struct Table *table_init(int width, int length, int flags);
/* MODULAR ACCESSIBLE */ int table_add(struct Table *table, char **entry);
/* MODULAR ACCESSIBLE */ int table_change(struct Table *table, int row, char **entry);
/* MODULAR ACCESSIBLE */ int table_change_field(struct Table *table, int row, int col, char *entry);
/* MODULAR ACCESSIBLE */ int table_set_bold(struct Table *table, int collum, int bold);
/* MODULAR ACCESSIBLE */ char **table_end(struct Table *table);
/* MODULAR ACCESSIBLE */ void table_free(struct Table *table);

/* MODULAR ACCESSIBLE */ char* timeToStr(struct UserNode *user, int seconds, int items, char *buf);
/* MODULAR ACCESSIBLE */ int strToTime(struct UserNode *user, char *str);

/* MODULAR ACCESSIBLE */ struct ModeBuffer* initModeBuffer(struct ClientSocket *client, struct ChanNode *chan);
/* MODULAR ACCESSIBLE */ void modeBufferSet(struct ModeBuffer *modeBuf, int add, char mode, char *param);
/* MODULAR ACCESSIBLE */ void flushModeBuffer(struct ModeBuffer *modeBuf);
/* MODULAR ACCESSIBLE */ void freeModeBuffer(struct ModeBuffer *modeBuf);

/* MODULAR ACCESSIBLE */ int is_ircmask(const char *text);

/* MODULAR ACCESSIBLE */ char* generate_banmask(struct UserNode *user, char *buffer); 
/* MODULAR ACCESSIBLE */ char* make_banmask(char *input, char* buffer);
/* MODULAR ACCESSIBLE */ int isFakeHost(char *host);

/* MODULAR ACCESSIBLE */ int mask_match(char *mask, struct UserNode *user);

/* MODULAR ACCESSIBLE */ unsigned long crc32(const char *text);

void init_tools();

#endif
#endif
