#! /bin/sh
echo "Extracting version.c ..."

if test -r version.c
then
   compilation=`sed -n 's/^const char \*compilation = \"\(.*\)\";/\1/p' < version.c`
   if test ! "$compilation" ; then compilation=0; fi
else
   compilation=0
fi

compilation=`expr $compilation + 1`

creation=`date | \
awk '{if (NF == 6) \
	 { print $1 " " $2 " " $3 " " $6 " at " $4 " " $5 } \
else \
	 { print $1 " " $2 " " $3 " " $7 " at " $4 " " $5 " " $6 }}'`

codelines=`find . -type f -regex '\./.*\.h' -or -regex '\./.*\.c' |xargs cat|wc -l`


git_revision=`git rev-list -n 1 --pretty="format:%h" --header master | grep '^[0-9a-f]*$'`
if test "x$git_revision" = "x" ; then
  git_revision="0"
  git_commitcount="0"
else
  git_commitcount=`git rev-list --oneline --header master | wc -l | sed "s/[ \t]//g"`
  git_revision="git-$git_commitcount-$git_revision"
fi


/bin/cat > version.c <<!SUB!THIS!
/* version.c - NeonServ v5.5
 * Copyright (C) 2011-2012  Philipp Kreil (pk910)
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program. If not, see <http://www.gnu.org/licenses/>. 
 */
//Auto generated file!

#include "version.h"

const char *compilation = "$compilation";
const char *creation = "$creation";
const char *revision = "$git_revision";
const char *codelines = "$codelines";

const int patchlevel = ($git_commitcount ? ($git_commitcount - VERSION_PATCHLEVEL) : 0);

const char *get_compilation() {
    return compilation;
}

const char *get_creation() {
    return creation;
}

const char *get_revision() {
    return revision;
}

const char *get_codelines() {
    return codelines;
}

const int get_patchlevel() {
    return patchlevel;
}

!SUB!THIS!

